/*---------------------------------------------------------------------*/
/* --- STC MCU Limited ------------------------------------------------*/
/* --- STC 1T Series MCU Demo Programme -------------------------------*/
/* --- Mobile: (86)13922805190 ----------------------------------------*/
/* --- Fax: 86-0513-55012956,55012947,55012969 ------------------------*/
/* --- Tel: 86-0513-55012928,55012929,55012966 ------------------------*/
/* --- Web: www.STCAI.com ---------------------------------------------*/
/* --- BBS: www.STCAIMCU.com  -----------------------------------------*/
/* --- QQ:  800003751 -------------------------------------------------*/
/* 如果要在程序中使用此代码,请在程序中注明使用了STC的资料及程序        */
/*---------------------------------------------------------------------*/

/*************  功能说明    **************

本例程基于STC USB转双串口核心功能实验板进行编写测试。

代码来源与STC官方论坛，USB版块，从STC32G平台移植到STC8H8K64U

由论坛荣誉版主，“电子DIY小家（冲哥）”大神发布，供学习交流使用

论坛原帖链接：
https://www.stcaimcu.com/forum.php?mod=viewthread&tid=4080&extra=page%3D1

下载时, 设置IRC工作频率为 24MHz.

******************************************/

#include "stc.h"
#include "usb.h"
#include "uart.h"
#include "EEPROM.h"
#include <string.h>
#include "timer.h"

static void delay(){
    int i;
    for (i = 0; i < 3000; i++){
        _nop_();
        _nop_();
        _nop_();
        _nop_();
    }
}

void sys_init(){
    P2M0 = 0xff;
    P2M1 = 0x00;
    P_SW2 |= 0x80;
    timer_init();
    usb_init();
    uart_init();
    EA = 1;
}

void usb_key_send(BYTE key[9]){
	BYTE i;
	IE2 &= ~0x80;
	if (usb_read_reg(POWER) & 0x02){
		usb_write_reg(POWER, 5);
		delay();
		usb_write_reg(POWER, 1);
	}
	UsbInBusy_HID = 1;
	usb_write_reg(INDEX, 1);
	for (i=0; i<9; i++){
		usb_write_reg(FIFO1, key[i]);
	}
	usb_write_reg(INCSR1, INIPRDY);
	IE2 |= 0x80;
}


BYTE bKeyCode = 0;
BYTE bKeyDebounce = 0;
BOOL fKeyOK;
BYTE bEc11Code = 0;
BYTE bEc11Debounce = 0;
BOOL fEc11OK;

BYTE key0[9] = {2, 0xB6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
BYTE key1[9] = {2, 0xCD, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
BYTE key2[9] = {2, 0xB5, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
BYTE key3[9] = {1, 0x05, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00};
BYTE key4[9] = {2, 0xE9, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
BYTE key5[9] = {2, 0xEA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

void EEPROM_update(){
    BYTE i;
	i = IapRead(0x0000);
	if(i != 0xff){
		for(i=0; i<9; i++){
			key0[i] = IapRead(0x0000+i);
		}
	}

	i = IapRead(0x0200);
	if(i != 0xff){
		for(i=0; i<9; i++){
			key1[i] = IapRead(0x0200+i);
		}
	}

	i = IapRead(0x0400);
	if(i != 0xff){
		for(i=0; i<9; i++){
			key2[i] = IapRead(0x0400+i);
		}
	}

	i = IapRead(0x0600);
	if(i != 0xff){
		for(i=0; i<9; i++){
			key3[i] = IapRead(0x0600+i);
		}
	}

	i = IapRead(0x0800);
	if(i != 0xff){
		for(i=0; i<9; i++){
			key4[i] = IapRead(0x0800+i);
		}
	}

	i = IapRead(0x0A00);
	if(i != 0xff){
		for(i=0; i<9; i++){
			key5[i] = IapRead(0x0A00+i);
		}
	}
}

void usb_class_in()
{
    static BYTE id = 0;
    BYTE key[9];
    
    if (DeviceState != DEVSTATE_CONFIGURED)
        return;

    if (!UsbInBusy && (fKeyOK || fEc11OK) ){

        key[0] = 1;
        key[1] = 0;
        key[2] = 0;
        key[3] = 0;
        key[4] = 0;
        key[5] = 0;
        key[6] = 0;
        key[7] = 0;
        key[8] = 0;

        if(fKeyOK){
            P2 = bKeyCode;
            switch (bKeyCode){
                case 0x01:
                    id = key0[0];
                    key[1] = key0[1];
                    key[2] = key0[2];
                    key[3] = key0[3];
                    key[4] = key0[4];
                    key[5] = key0[5];
                    key[6] = key0[6];
                    key[7] = key0[7];
                    key[8] = key0[8];
                    break;
                case 0x02:
                    id = key1[0];
                    key[1] = key1[1];
                    key[2] = key1[2];
                    key[3] = key1[3];
                    key[4] = key1[4];
                    key[5] = key1[5];
                    key[6] = key1[6];
                    key[7] = key1[7];
                    key[8] = key1[8];
                    break;
                case 0x03:
                    id = key2[0];
                    key[1] = key2[1];
                    key[2] = key2[2];
                    key[3] = key2[3];
                    key[4] = key2[4];
                    key[5] = key2[5];
                    key[6] = key2[6];
                    key[7] = key2[7];
                    key[8] = key2[8];
                    break;
                case 0x04:
                    id = key3[0];
                    key[1] = key3[1];
                    key[2] = key3[2];
                    key[3] = key3[3];
                    key[4] = key3[4];
                    key[5] = key3[5];
                    key[6] = key3[6];
                    key[7] = key3[7];
                    key[8] = key3[8];
                    break;
            }
        }

        if(fEc11OK){
            P2 = bEc11Code;
            switch (bEc11Code){
                case 0x05:
                    id = key4[0];
                    key[1] = key4[1];
                    key[2] = key4[2];
                    key[3] = key4[3];
                    key[4] = key4[4];
                    key[5] = key4[5];
                    key[6] = key4[6];
                    key[7] = key4[7];
                    key[8] = key4[8];
                    break;
                case 0x06:
                    id = key5[0];
                    key[1] = key5[1];
                    key[2] = key5[2];
                    key[3] = key5[3];
                    key[4] = key5[4];
                    key[5] = key5[5];
                    key[6] = key5[6];
                    key[7] = key5[7];
                    key[8] = key5[8];
                    break;
            }
        }

        key[0] = id;
        
        usb_key_send(key);
        fKeyOK = 0;
        fEc11OK = 0;
    }
}

void scan_key(){
    BYTE key;
    key = 0;

    P3M0 = 0x00;
    P3M1 = 0x00;

	if(P32==0)key = 0x01;
    if(P33==0)key = 0x02;
    if(P34==0)key = 0x03;
    if(P35==0)key = 0x04;

    //消抖
    if(key != bKeyCode){
        bKeyCode = key;
        bKeyDebounce = 20;
    }else{
        if(bKeyDebounce){
            bKeyDebounce--;
            if(bKeyDebounce == 0){
                fKeyOK = 1;
            }
        }
    }
}

void scan_ec11(){
    BYTE key;
    key = 0x00;
    P5M0 = 0x00;
    P5M1 = 0x00;
    P5PU = 0xff;
    P5 = 0xff;

	if(P51==0 && bEc11Debounce == 0){
        bEc11Debounce = 1;
    }
    if(bEc11Debounce > 0 && bEc11Debounce < 88){
        bEc11Debounce++;
    }
    if(bEc11Debounce == 3){
        if(P50==1){
            key = 0x05;
        }else{
            key = 0x06;
        }
        bEc11Code = key;
        fEc11OK = 1;
    }
    if(bEc11Debounce == 25){
        bEc11Code = 0x00;
        fEc11OK = 1;
    }
    if(P51 == 1 && P50 == 1 && fEc11OK == 0 && bEc11Debounce > 50){
        bEc11Debounce = 0;
    }

}

void main(){
	P_SW2 |= 0x80;  //扩展寄存器(XFR)访问使能

	P3M0 = 0X00;P3M1 = 0X00;
	P2M0 = 0xFF;P2M1 = 0X00;
	P2 = 0x00;
    sys_init();
    EEPROM_update();

    EA = 1;

    while (1){
        uart_polling();
		if (f1ms){
            f1ms = 0;
			scan_key();
			scan_ec11();
		}
        usb_class_in();
    }

}
